'use strict';

exports.__esModule = true;

var _mkI18n = require('mk-element-ui2/lib/datas/mk-i18n');

var _mkI18n2 = _interopRequireDefault(_mkI18n);

var _mkCacheUtil = require('mk-element-ui2/lib/utils/mk-cache-util');

var _lodash = require('lodash.isempty');

var _lodash2 = _interopRequireDefault(_lodash);

var _mutationTypes = require('mk-element-ui2/lib/store/mutation-types');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var install = function install(Vue, _ref) {
  var router = _ref.router,
      loginPath = _ref.loginPath,
      store = _ref.store;

  if (!router) return;
  /**
   * 重定向
   * @param ms
   * @param url false表示取消跳转
   * @param next
   * @returns {*}
   */
  var redirects = function redirects(_ref2) {
    var ms = _ref2.ms,
        url = _ref2.url,
        next = _ref2.next;

    if (ms) {
      Vue.prototype.$message({
        message: ms,
        type: 'warning'
      });
    }
    return next({
      path: url
    });
  };
  /**
   * 严格权限检测
   * @param path 目标路由
   * @returns {boolean}
   */
  var strictRole = function strictRole(path) {
    // store.state.ui 递归
    return true;
  };

  // 过渡效果自动关闭定时器
  router.beforeEach(function (to, from, next) {
    // 未知路由, 取消跳转
    if (to.name === null) {
      return redirects({ ms: _mkI18n2.default.routers.not_found, url: false, next: next });
    }
    // 当状态没有菜单用户信息时，尝试从cache中读取
    if ((0, _lodash2.default)(store.state.user) === true) {
      var ui = (0, _mkCacheUtil.get)(_mkCacheUtil.keys.menus);
      if (ui) {
        ui = JSON.parse(ui);
        store.commit(_mutationTypes.BIND_MENUS, ui);
      }
      var dataPermissions = (0, _mkCacheUtil.get)(_mkCacheUtil.keys.dataPermission);
      if (dataPermissions) {
        dataPermissions = JSON.parse(dataPermissions);
        store.commit(_mutationTypes.BIND_DATAPERMISSION, dataPermissions);
      }
      var user = (0, _mkCacheUtil.get)(_mkCacheUtil.keys.user);
      if (user) {
        user = JSON.parse(user);
        store.commit(_mutationTypes.BIND_USER, user);
      }
      var config = (0, _mkCacheUtil.get)(_mkCacheUtil.keys.config);
      if (config) {
        config = JSON.parse(config);
        store.commit(_mutationTypes.BIND_CONFIG, config);
      }
      var menu = (0, _mkCacheUtil.get)(_mkCacheUtil.keys.tabs);
      if (menu) {
        menu = JSON.parse(menu);
        store.commit(_mutationTypes.BIND_TABS, menu);
      }
      var uris = (0, _mkCacheUtil.get)(_mkCacheUtil.keys.uris);
      if (uris) {
        uris = JSON.parse(uris);
        store.commit(_mutationTypes.BIND_URIS, uris);
      }
    }

    // 目标路由不允许跳过登录信息检测，同时状态中没有用户信息时，跳转登录页面
    if (to.meta.skip !== true && (0, _lodash2.default)(store.state.user) === true) {
      return redirects({ url: loginPath, next: next });
    }
    // 目标路由需要检测用户是否具有访问权限，同时权限检测失败时，取消跳转
    if (to.meta.strict === true && strictRole(to.path) === false) {
      // 检测菜单
      return redirects({ ms: _mkI18n2.default.routers.no_role, url: loginPath, next: next });
    }

    // 绑定全局filters
    Vue.filter('dataConfig', function (value, configName) {
      return store.state.config.GlobalMap[configName][value];
    });
    // 检测通过
    next();
  });
  router.afterEach(function (to, from) {});
};
exports.default = {
  install: install
};