'use strict';

exports.__esModule = true;

var _decimal = require('decimal.js');

var _decimal2 = _interopRequireDefault(_decimal);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var isNum = function isNum(a) {
  return a !== '' && a !== null && !isNaN(a);
};

var filterNumber = function filterNumber(array) {
  var res = [];
  array.forEach(function (element) {
    if (isNum(element)) {
      res.push(+element);
    }
  });
  return res;
};

var _max = function _max() {
  for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  var arr = filterNumber(args);
  return Math.max.apply(null, arr);
};

var _min = function _min() {
  for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    args[_key2] = arguments[_key2];
  }

  var arr = filterNumber(args);
  return Math.min.apply(null, arr);
};

var _div = function _div() {
  for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
    args[_key3] = arguments[_key3];
  }

  var arr = filterNumber(args);
  if (arr.length > 1) {
    return arr.reduce(function (a, b) {
      return new _decimal2.default(a).div(b);
    }).toNumber();
  } else if (arr.length > 0) {
    return new _decimal2.default(arr[0]).toNumber();
  } else {
    return 0;
  }
};

var _add = function _add() {
  for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
    args[_key4] = arguments[_key4];
  }

  var arr = filterNumber(args);
  if (arr.length > 1) {
    return arr.reduce(function (a, b) {
      return new _decimal2.default(a).add(b);
    }).toNumber();
  } else if (arr.length > 0) {
    return new _decimal2.default(arr[0]).toNumber();
  } else {
    return 0;
  }
};

var _mul = function _mul() {
  for (var _len5 = arguments.length, args = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
    args[_key5] = arguments[_key5];
  }

  var arr = filterNumber(args);
  if (arr.length > 1) {
    return arr.reduce(function (a, b) {
      return new _decimal2.default(a).mul(b);
    }).toNumber();
  } else if (arr.length > 0) {
    return new _decimal2.default(arr[0]).toNumber();
  } else {
    return 0;
  }
};

var _sub = function _sub() {
  for (var _len6 = arguments.length, args = Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
    args[_key6] = arguments[_key6];
  }

  var arr = filterNumber(args);
  if (arr.length > 1) {
    return arr.reduce(function (a, b) {
      return new _decimal2.default(a).sub(b);
    }).toNumber();
  } else if (arr.length > 0) {
    return new _decimal2.default(arr[0]).toNumber();
  } else {
    return 0;
  }
};

var roundMode = [_decimal2.default.ROUND_HALF_UP, _decimal2.default.ROUND_UP, _decimal2.default.ROUND_DOWN];
var _round = function _round(number, numDigits) {
  var upOrDown = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

  if (isNum(number)) {
    var nu = new _decimal2.default(number);
    return nu.toDP(numDigits, roundMode[upOrDown]).toNumber();
  } else {
    return 0;
  }
};

var _money = function _money(value) {
  var prefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 2;
  var def = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

  var num = +value;
  return (isNum(num) ? num : def).toLocaleString('zh', {
    style: 'currency',
    currency: 'CNY',
    minimumFractionDigits: prefix
  });
};

var calc = function calc() {
  var start = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;

  var result = start;
  var bus = {
    add: function add() {
      for (var _len7 = arguments.length, args = Array(_len7), _key7 = 0; _key7 < _len7; _key7++) {
        args[_key7] = arguments[_key7];
      }

      result = _add.apply(undefined, [result].concat(args));
      return bus;
    },
    sub: function sub() {
      for (var _len8 = arguments.length, args = Array(_len8), _key8 = 0; _key8 < _len8; _key8++) {
        args[_key8] = arguments[_key8];
      }

      result = _sub.apply(undefined, [result].concat(args));
      return bus;
    },
    div: function div() {
      for (var _len9 = arguments.length, args = Array(_len9), _key9 = 0; _key9 < _len9; _key9++) {
        args[_key9] = arguments[_key9];
      }

      result = _div.apply(undefined, [result].concat(args));
      return bus;
    },
    mul: function mul() {
      for (var _len10 = arguments.length, args = Array(_len10), _key10 = 0; _key10 < _len10; _key10++) {
        args[_key10] = arguments[_key10];
      }

      result = _mul.apply(undefined, [result].concat(args));
      return bus;
    },
    max: function max() {
      for (var _len11 = arguments.length, args = Array(_len11), _key11 = 0; _key11 < _len11; _key11++) {
        args[_key11] = arguments[_key11];
      }

      result = _max.apply(undefined, [result].concat(args));
      return bus;
    },
    min: function min() {
      for (var _len12 = arguments.length, args = Array(_len12), _key12 = 0; _key12 < _len12; _key12++) {
        args[_key12] = arguments[_key12];
      }

      result = _min.apply(undefined, [result].concat(args));
      return bus;
    },
    round: function round(numDigits, upOrDown) {
      result = _round(result, numDigits, upOrDown);
      return bus;
    },
    over: function over() {
      return result;
    },
    money: function money(prefix, def) {
      return _money(result, prefix, def);
    }
  };
  return bus;
};

exports.default = {
  /**
   * 流式计算
   * 终结方式1
   * calc(5).add(1,2,3).sub(4,5,6).div(80).mul(10,20).round(6,2).over()
   * 终结方式2
   * calc(7.99).add(1, 2, 3).money(2)
   */
  calc: calc,
  /**
   * 作者 冯浩
   * 说明 在数字类型上加了自己实现的四舍五入方法
   * @param tofixednum 要处理的数字
   * @param numDigits 保留几位小数，为大于等于0的整数,如果输入小数则丢弃小数部分,保留整数部分,如果输入非数字或负数则视为0
   * @param upOrDown 入或舍,当0----四舍五入,1----入,2----舍,不传默认四舍五入.
   * @returns 返回四舍五入后的结果
   */
  round: _round,
  /**
   * 减法运算 arg1-arg2
   * @param arg1 this 被减数
   * @param arg2 减数
   * @returns {string}运算结果
   * @author 冯浩
   * @create date 2016/04/19
   */
  sub: _sub,
  /**
   * 除法运算 arg1÷arg2
   * @param arg1 被除数
   * @param arg2 除数
   * @returns {number}运算结果
   * @author 冯浩
   * @create date 2016/04/19
   */
  div: _div,
  /**
   * 乘法运算 多个参数
   * @returns {number}运算结果
   * @author 冯浩
   * @create date 2016/04/19
   */
  mul: _mul,
  /**
   * 加法运算 乘法运算 多个参数 减法传负数
   * @returns {number}运算结果
   * @author 冯浩
   * @create date 2016/04/19
   */
  add: _add,
  /**
   * 金额格式化
   */
  money: _money,
  max: _max,
  min: _min,
  /** 是否是数字 */
  isNum: isNum
};